@@ -51,7 +51,7 @@ static void (*cleanup_fnc)(void);
 
 #ifndef HAVE_DOSISH_SYSTEM
 static void
-init_one_signal (int sig, RETSIGTYPE (*handler)(int), int check_ign )
+init_one_signal (int sig, void (*handler)(int), int check_ign )
 {
 # ifdef HAVE_SIGACTION
   struct sigaction oact, nact;
@@ -69,7 +69,7 @@ init_one_signal (int sig, RETSIGTYPE (*handler)(int), int check_ign )
   nact.sa_flags = 0;
   sigaction ( sig, &nact, NULL);
 # else
-  RETSIGTYPE (*ohandler)(int);
+  void (*ohandler)(int);
 
   ohandler = signal (sig, handler);
   if (check_ign && ohandler == SIG_IGN)
@@ -87,8 +87,14 @@ get_signal_name( int signum )
 {
   /* Note that we can't use strsignal(), because it is not
      reentrant. */
-#if HAVE_DECL_SYS_SIGLIST && defined(NSIG)
-  return (signum >= 0 && signum < NSIG) ? sys_siglist[signum] : "?";
+#if HAVE_SIGDESCR_NP
+  return sigdescr_np (signum);
+#elif (HAVE_DECL_SYS_SIGLIST || HAVE_DECL__SYS_SIGLIST) && defined(NSIG)
+#if HAVE_DECL_SYS_SIGLIST
+#undef _sys_siglist
+#define _sys_siglist sys_siglist
+#endif
+  return (signum >= 0 && signum < NSIG) ? _sys_siglist[signum] : "?";
 #else
   return NULL;
 #endif
@@ -96,7 +102,7 @@ get_signal_name( int signum )
 #endif /*!HAVE_DOSISH_SYSTEM*/
 
 #ifndef HAVE_DOSISH_SYSTEM
-static RETSIGTYPE
+static void
 got_fatal_signal (int sig)
 {
   const char *s;
@@ -155,7 +161,7 @@ got_fatal_signal (int sig)
 #endif /*!HAVE_DOSISH_SYSTEM*/
 
 #ifndef HAVE_DOSISH_SYSTEM
-static RETSIGTYPE
+static void
 got_usr_signal (int sig)
 {
   (void)sig;
@@ -237,13 +243,13 @@ do_block (int block)
 
 
 void
-gnupg_block_all_signals ()
+gnupg_block_all_signals (void)
 {
   do_block(1);
 }
 
 void
-gnupg_unblock_all_signals ()
+gnupg_unblock_all_signals (void)
 {
   do_block(0);
 }
