@@ -1,5 +1,5 @@
 /* call-keyboxd.c - Access to the keyboxd storage server
- * Copyright (C) 2019  g10 Code GmbH
+ * Copyright (C) 2019, 2024  g10 Code GmbH
  *
  * This file is part of GnuPG.
  *
@@ -38,7 +38,6 @@
 #include "../common/i18n.h"
 #include "../common/asshelp.h"
 #include "../common/host2net.h"
-#include "../common/exechelp.h"
 #include "../common/status.h"
 #include "../kbx/kbx-client-util.h"
 #include "keydb.h"
@@ -327,20 +326,6 @@ keydb_release (KEYDB_HANDLE hd)
 }
 
 
-/* Take a lock if we are not using the keyboxd.  */
-gpg_error_t
-keydb_lock (KEYDB_HANDLE hd)
-{
-  if (!hd)
-    return gpg_error (GPG_ERR_INV_ARG);
-
-  if (!hd->use_keyboxd)
-    return internal_keydb_lock (hd);
-
-  return 0;
-}
-
-
 /* Return the keyblock last found by keydb_search() in *RET_KB.
  *
  * On success, the function returns 0 and the caller must free *RET_KB
@@ -718,7 +703,7 @@ keydb_search (KEYDB_HANDLE hd, KEYDB_SEARCH_DESC *desc,
   if (DBG_CLOCK)
     log_clock ("%s enter", __func__);
 
-  if (DBG_LOOKUP)
+  if (DBG_KEYDB)
     {
       log_debug ("%s: %zu search descriptions:\n", __func__, ndesc);
       for (i = 0; i < ndesc; i ++)
@@ -881,7 +866,7 @@ keydb_search (KEYDB_HANDLE hd, KEYDB_SEARCH_DESC *desc,
     {
       hd->kbl->search_result = iobuf_temp_with_content (buffer, len);
       xfree (buffer);
-      if (DBG_LOOKUP && hd->last_ubid_valid)
+      if (DBG_KEYDB && hd->last_ubid_valid)
         log_printhex (hd->last_ubid, 20, "found UBID (%d,%d):",
                       hd->last_uid_no, hd->last_pk_no);
     }
