@@ -38,6 +38,7 @@
 #include "filter.h"
 #include "../common/ttyio.h"
 #include "../common/i18n.h"
+#include "../common/shareddefs.h"
 #include "call-agent.h"
 
 
@@ -65,16 +66,22 @@ do_delete_key (ctrl_t ctrl, const char *username, int secret, int force,
 
   *r_sec_avail = 0;
 
-  hd = keydb_new ();
+  hd = keydb_new (ctrl);
   if (!hd)
     return gpg_error_from_syserror ();
 
   /* Search the userid.  */
   err = classify_user_id (username, &desc, 1);
-  exactmatch = (desc.mode == KEYDB_SEARCH_MODE_FPR
-                || desc.mode == KEYDB_SEARCH_MODE_FPR16
-                || desc.mode == KEYDB_SEARCH_MODE_FPR20);
+  exactmatch = (desc.mode == KEYDB_SEARCH_MODE_FPR);
   thiskeyonly = desc.exact;
+
+  err = keydb_lock (hd);
+  if (err)
+    {
+      keydb_release (hd);
+      goto leave;
+    }
+
   if (!err)
     err = keydb_search (hd, &desc, 1, NULL);
   if (err)
@@ -133,7 +140,7 @@ do_delete_key (ctrl_t ctrl, const char *username, int secret, int force,
 
   if (!secret && !force)
     {
-      if (have_secret_key_with_kid (keyid))
+      if (have_secret_key_with_kid (ctrl, keyid))
         {
           *r_sec_avail = 1;
           err = gpg_error (GPG_ERR_EOF);
@@ -143,7 +150,7 @@ do_delete_key (ctrl_t ctrl, const char *username, int secret, int force,
         err = 0;
     }
 
-  if (secret && !have_secret_key_with_kid (keyid))
+  if (secret && !have_secret_key_with_kid (ctrl, keyid))
     {
       err = gpg_error (GPG_ERR_NOT_FOUND);
       log_error (_("key \"%s\" not found\n"), username);
@@ -153,7 +160,13 @@ do_delete_key (ctrl_t ctrl, const char *username, int secret, int force,
 
 
   if (opt.batch && exactmatch)
-    okay++;
+    {
+      if (secret && opt.pinentry_mode == PINENTRY_MODE_LOOPBACK
+          && !opt.answer_yes)
+        log_error(_("can't do this in batch mode without \"--yes\"\n"));
+      else
+        okay++;
+    }
   else if (opt.batch && secret)
     {
       log_error(_("can't do this in batch mode\n"));
@@ -168,12 +181,8 @@ do_delete_key (ctrl_t ctrl, const char *username, int secret, int force,
     }
   else
     {
-      if (secret)
-        print_seckey_info (ctrl, pk);
-      else
-        print_pubkey_info (ctrl, NULL, pk );
-      tty_printf( "\n" );
-
+      print_key_info (ctrl, NULL, 0, pk, secret);
+      tty_printf ("\n");
       if (thiskeyonly == 1 && !secret)
         {
           /* We need to delete the entire public key despite the use
@@ -238,7 +247,7 @@ do_delete_key (ctrl_t ctrl, const char *username, int secret, int force,
               if (thiskeyonly && targetnode != node)
                 continue;
 
-              if (agent_probe_secret_key (NULL, node->pkt->pkt.public_key))
+              if (!agent_probe_secret_key (NULL, node->pkt->pkt.public_key))
                 continue;  /* No secret key for that public (sub)key.  */
 
               prompt = gpg_format_keydesc (ctrl,
@@ -281,25 +290,16 @@ do_delete_key (ctrl_t ctrl, const char *username, int secret, int force,
 	}
       else if (thiskeyonly == 2)
         {
-          int selected = 0;
-
           /* Delete the specified public subkey.  */
           for (kbctx=NULL; (node = walk_kbnode (keyblock, &kbctx, 0)); )
-            {
-              if (thiskeyonly && targetnode != node)
-                continue;
+            if (targetnode == node)
+              break;
+          log_assert (node);
+          delete_kbnode (node);
+          while ((node = walk_kbnode (keyblock, &kbctx, 0))
+                 && node->pkt->pkttype == PKT_SIGNATURE)
+            delete_kbnode (node);
 
-              if (node->pkt->pkttype == PKT_PUBLIC_SUBKEY)
-                {
-                  selected = targetnode == node;
-                  if (selected)
-                    delete_kbnode (node);
-                }
-              else if (selected && node->pkt->pkttype == PKT_SIGNATURE)
-                delete_kbnode (node);
-              else
-                selected = 0;
-            }
           commit_kbnode (&keyblock);
           err = keydb_update_keyblock (ctrl, hd, keyblock);
           if (err)
