@@ -337,6 +337,10 @@ gpgsm_add_to_certlist (ctrl_t ctrl, const char *name, int secret,
   KEYDB_SEARCH_DESC desc;
   KEYDB_HANDLE kh = NULL;
   ksba_cert_t cert = NULL;
+  ksba_isotime_t current_time;
+  ksba_isotime_t exp_time = {0};
+
+  gnupg_get_isotime (current_time);
 
   rc = classify_user_id (name, &desc, 0);
   if (!rc)
@@ -347,6 +351,7 @@ gpgsm_add_to_certlist (ctrl_t ctrl, const char *name, int secret,
       else
         {
           int wrong_usage = 0;
+          int expired_rc = 0;
           char *first_subject = NULL;
           char *first_issuer = NULL;
 
@@ -365,10 +370,14 @@ gpgsm_add_to_certlist (ctrl_t ctrl, const char *name, int secret,
                 }
               rc = secret? gpgsm_cert_use_sign_p (cert, 0)
                          : gpgsm_cert_use_encrypt_p (cert);
+              if (!rc)
+                rc = check_validity_period_cm (current_time, current_time,
+                                               cert, exp_time, 0, NULL, 0, 0);
+
               if (gpg_err_code (rc) == GPG_ERR_WRONG_KEY_USAGE)
                 {
                   /* There might be another certificate with the
-                     correct usage, so we try again */
+                   * correct usage, so we try again */
                   if (!wrong_usage
                       || same_subject_issuer (first_subject, first_issuer,cert))
                     {
@@ -381,12 +390,23 @@ gpgsm_add_to_certlist (ctrl_t ctrl, const char *name, int secret,
                     }
                   else
                     wrong_usage = rc;
-
+                }
+              else if (gpg_err_code (rc) == GPG_ERR_CERT_EXPIRED
+                       || gpg_err_code (rc) == GPG_ERR_CERT_TOO_YOUNG)
+                {
+                  if (!expired_rc)
+                    expired_rc = rc;
+                  ksba_cert_release (cert);
+                  cert = NULL;
+                  log_info (_("looking for another certificate\n"));
+                  goto get_next;
                 }
             }
           /* We want the error code from the first match in this case. */
           if (rc && wrong_usage)
             rc = wrong_usage;
+          else if (rc && expired_rc)
+            rc = expired_rc;
 
           if (!rc)
             {
@@ -416,21 +436,41 @@ gpgsm_add_to_certlist (ctrl_t ctrl, const char *name, int secret,
                      keybox).  */
                   if (!keydb_get_cert (kh, &cert2))
                     {
-                      int tmp = (same_subject_issuer (first_subject,
-                                                      first_issuer,
-                                                      cert2)
-                                 && ((gpg_err_code (
-                                      secret? gpgsm_cert_use_sign_p (cert2,0)
-                                            : gpgsm_cert_use_encrypt_p (cert2)
-                                      )
-                                     )  == GPG_ERR_WRONG_KEY_USAGE));
+                      gpg_err_code_t tmp;
+
+                      if (same_subject_issuer (first_subject,
+                                               first_issuer,
+                                               cert2))
+                        {
+                          tmp = gpg_err_code (
+                                   secret? gpgsm_cert_use_sign_p (cert2, 0)
+                                         : gpgsm_cert_use_encrypt_p (cert2)
+                                                ) == GPG_ERR_WRONG_KEY_USAGE;
+                          if (!tmp)
+                            {
+                              switch (gpg_err_code (
+                                  check_validity_period_cm (current_time,
+                                                            current_time,
+                                                            cert,
+                                                            exp_time,
+                                                            0, NULL, 0, 1)))
+                                {
+                                case GPG_ERR_CERT_EXPIRED:
+                                case GPG_ERR_CERT_TOO_YOUNG: tmp = 1; break;
+                                default: tmp = 0; break;
+                                }
+                            }
+                        }
+                      else
+                        tmp = 0;
+
                       if (tmp)
                         gpgsm_add_cert_to_certlist (ctrl, cert2,
                                                     &dup_certs, 0);
                       else
                         {
                           if (is_cert_in_certlist (cert2, dup_certs))
-                            tmp = 1;
+                            tmp = GPG_ERR_TRUE;
                         }
 
                       ksba_cert_release (cert2);
@@ -448,6 +488,11 @@ gpgsm_add_to_certlist (ctrl_t ctrl, const char *name, int secret,
 
           if (!rc && !is_cert_in_certlist (cert, *listaddr))
             {
+              unsigned int valflags = 0;
+
+              if (!secret && (opt.always_trust || ctrl->always_trust))
+                valflags |= VALIDATE_FLAG_BYPASS;
+
               if (!rc && secret)
                 {
                   char *p;
@@ -461,9 +506,10 @@ gpgsm_add_to_certlist (ctrl_t ctrl, const char *name, int secret,
                       xfree (p);
                     }
                 }
+
               if (!rc)
                 rc = gpgsm_validate_chain (ctrl, cert, GNUPG_ISOTIME_NONE, NULL,
-                                           0, NULL, 0, NULL);
+                                           0, NULL, valflags, NULL);
               if (!rc)
                 {
                   certlist_t cl = xtrycalloc (1, sizeof *cl);
@@ -563,6 +609,7 @@ gpgsm_find_cert (ctrl_t ctrl,
               ksba_isotime_t notbefore = "";
               const unsigned char *image = NULL;
               size_t length = 0;
+
               if (allow_ambiguous)
                 {
                   /* We want to return the newest certificate */
