@@ -17,7 +17,8 @@
 ;; You should have received a copy of the GNU General Public License
 ;; along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-(load (with-path "defs.scm"))
+(load (in-srcdir "tests" "openpgp" "defs.scm"))
+(setup-legacy-environment)
 
 (for-each-p
  "Checking armored signatures"
@@ -25,6 +26,6 @@
    (tr:do
     (tr:open source)
     (tr:gpg usrpass1 `(--yes --passphrase-fd "0" -sa --recipient ,usrname2))
-    (tr:gpg "" '(--yes))
+    (tr:gpg "" '(--yes --decrypt))
     (tr:assert-identity source)))
  (append plain-files data-files))
