@@ -212,11 +212,9 @@ my_strusage( int level )
 static void
 wrong_args (const char *text, const char *text2)
 {
-#if GPGRT_VERSION_NUMBER >= 0x013000 /* >= 1.48 */
   /* Skip the leading dashes if build with command support.  */
   if (text[0] == '-' && text[1] == '-' && text[2])
     text += 2;
-#endif
   es_fprintf (es_stderr, _("usage: %s %s [options] %s\n"),
               gpgrt_strusage (11), text, text2);
   exit (2);
@@ -330,9 +328,7 @@ main (int argc, char **argv)
   pargs.argc  = &argc;
   pargs.argv  = &argv;
   pargs.flags = ARGPARSE_FLAG_KEEP;
-#if GPGRT_VERSION_NUMBER >= 0x013000 /* >= 1.48 */
   pargs.flags |= ARGPARSE_FLAG_COMMAND;
-#endif
   cmd = parse_arguments (&pargs, opts);
   gpgrt_argparse (NULL, &pargs, NULL);
 
@@ -1453,7 +1449,7 @@ command_create (const char *fingerprint, const char *userid)
         }
       key = NULL;
       err = mime_maker_add_body_data (mime, data, datalen);
-      xfree (data);
+      es_free (data);
       if (err)
         goto leave;
     }
