@@ -86,6 +86,7 @@ struct part
   struct part *down;      /* A contained part. */
   HDR_LINE hdr_lines;       /* Header lines os that part. */
   HDR_LINE *hdr_lines_tail; /* Helper for adding lines. */
+  const char *last_hdr_line;/* NULL or a ptr to the last inserted hdr.  */
   char *boundary;           /* Only used in the first part. */
 };
 typedef struct part *part_t;
@@ -99,6 +100,8 @@ struct rfc822parse_context
   int in_preamble;      /* Whether we are before the first boundary. */
   part_t parts;         /* The tree of parts. */
   part_t current_part;  /* Whom we are processing (points into parts). */
+  part_t this_part;     /* Same as current_part but kept until new
+                         * headers are processed.  */
   const char *boundary; /* Current boundary. */
 };
 
@@ -146,19 +149,21 @@ my_toupper (int c)
   return c;
 }
 
-/* This is the same as ascii_strcasecmp.  */
+/* This is like ascii_strcasecmp but stops at the first colon and
+ * returns true if A and B are the same.  */
 static int
-my_strcasecmp (const char *a, const char *b)
+same_header_name (const char *a, const char *b)
 {
   if (a == b)
-    return 0;
+    return 1;
 
-  for (; *a && *b; a++, b++)
-    {
-      if (*a != *b && my_toupper(*a) != my_toupper(*b))
-        break;
-    }
-  return *a == *b? 0 : (my_toupper (*a) - my_toupper (*b));
+  for (; *a && *a != ':' && *b && *b != ':'
+         && (*a == *b || my_toupper(*a) == my_toupper(*b)); a++,b++)
+    ;
+  if ((!*a || *a == ':') && (!*b || *b == ':'))
+    return 1;
+
+  return 0;
 }
 
 
@@ -233,10 +238,20 @@ release_handle_data (rfc822parse_t msg)
   release_part (msg->parts);
   msg->parts = NULL;
   msg->current_part = NULL;
+  msg->this_part = NULL;
   msg->boundary = NULL;
 }
 
 
+/* Wrapper around free becuase in this moulde we use a plain free.  */
+void
+rfc822_free (void *a)
+{
+  if (a)
+    free (a);
+}
+
+
 /* Check that the header name is valid.  We allow all lower and
  * uppercase letters and, except for the first character, digits and
  * the dash.  The check stops at the first colon or at string end.
@@ -269,9 +284,9 @@ rfc822_capitalize_header_name (char *name)
   int first = 1;
 
   /* Special cases first.  */
-  if (!my_strcasecmp (name, "MIME-Version"))
+  if (same_header_name (name, "MIME-Version"))
     {
-      strcpy (name, "MIME-Version");
+      memcpy (name, "MIME-Version", 12);
       return;
     }
 
@@ -292,6 +307,22 @@ rfc822_capitalize_header_name (char *name)
 }
 
 
+/* This is an strcmp which considers a colon also as end-of-string.
+ * Use this function to compare capitalized header names.  */
+int
+rfc822_cmp_header_name (const char *a, const char *b)
+{
+  for (; *a && *a != ':' && *b && *b != ':'; a++, b++)
+    {
+      if (*a != *b )
+        break;
+    }
+  if (*a == *b || (!*a && *b == ':') || (!*b && *a == ':'))
+    return 0;
+  else
+    return (*(signed char *)a - *(signed char *)b);
+}
+
 
 /* Create a new parsing context for an entire rfc822 message and
    return it.  CB and CB_VALUE may be given to callback for certain
@@ -302,7 +333,7 @@ rfc822parse_open (rfc822parse_cb_t cb, void *cb_value)
   rfc822parse_t msg = calloc (1, sizeof *msg);
   if (msg)
     {
-      msg->parts = msg->current_part = new_part ();
+      msg->parts = msg->current_part = msg->this_part = new_part ();
       if (!msg->parts)
         {
           free (msg);
@@ -385,7 +416,7 @@ set_current_part_to_parent (rfc822parse_t msg)
     assert (part);
   }
 #endif
-  msg->current_part = parent;
+  msg->current_part = msg->this_part = parent;
 
   parent = find_parent (msg->parts, parent);
   msg->boundary = parent? parent->boundary: NULL;
@@ -488,6 +519,7 @@ static int
 insert_header (rfc822parse_t msg, const unsigned char *line, size_t length)
 {
   HDR_LINE hdr;
+  int new_hdr = 0;
 
   if (!msg->current_part)
     {
@@ -515,15 +547,25 @@ insert_header (rfc822parse_t msg, const unsigned char *line, size_t length)
 
   /* Transform a field name into canonical format. */
   if (!hdr->cont && strchr (line, ':'))
-    rfc822_capitalize_header_name (hdr->line);
+    {
+      rfc822_capitalize_header_name (hdr->line);
+      msg->current_part->last_hdr_line = hdr->line;
+      new_hdr = 1;
+    }
+  else
+    msg->current_part->last_hdr_line = NULL;
 
   *msg->current_part->hdr_lines_tail = hdr;
   msg->current_part->hdr_lines_tail = &hdr->next;
 
+  if (new_hdr)
+    do_callback (msg, RFC822PARSE_HEADER_SEEN);
   /* Lets help the caller to prevent mail loops and issue an event for
    * every Received header. */
   if (length >= 9 && !memcmp (line, "Received:", 9))
      do_callback (msg, RFC822PARSE_RCVD_SEEN);
+
+  msg->this_part = msg->current_part;
   return 0;
 }
 
@@ -588,6 +630,18 @@ rfc822parse_finish (rfc822parse_t msg)
 }
 
 
+/* If the last inserted line was a header and not a continuation of a
+ * header line, return a pointer to that line.  This function may be
+ * used on the RFC822PARSE_HEADER_SEEN event to get the name of the
+ * current header.  Returns NULL if no header is available.  */
+const char *
+rfc822parse_last_header_line (rfc822parse_t msg)
+{
+  if (!msg || !msg->current_part)
+    return NULL;
+  return  msg->current_part->last_hdr_line;
+}
+
 
 /****************
  * Get a copy of a header line. The line is returned as one long
@@ -675,10 +729,10 @@ rfc822parse_enum_header_lines (rfc822parse_t msg, void **context)
   if (!msg) /* Close. */
     return NULL;
 
-  if (*context == msg || !msg->current_part)
+  if (*context == msg || !msg->this_part)
     return NULL;
 
-  l = *context ? (HDR_LINE) *context : msg->current_part->hdr_lines;
+  l = *context ? (HDR_LINE) *context : msg->this_part->hdr_lines;
 
   if (l)
     {
@@ -697,7 +751,7 @@ rfc822parse_enum_header_lines (rfc822parse_t msg, void **context)
  *
  *  which  -1 : Retrieve the last field
  *	   >0 : Retrieve the n-th field
-
+ *
  * RPREV may be used to return the predecessor of the returned field;
  * which may be NULL for the very first one. It has to be initialized
  * to either NULL in which case the search start at the first header line,
@@ -712,7 +766,7 @@ find_header (rfc822parse_t msg, const char *name, int which, HDR_LINE *rprev)
   int found = 0;
   int glob = 0;
 
-  if (!msg->current_part)
+  if (!msg->this_part)
     return NULL;
 
   namelen = strlen (name);
@@ -722,7 +776,7 @@ find_header (rfc822parse_t msg, const char *name, int which, HDR_LINE *rprev)
       glob = 1;
     }
 
-  hdr = msg->current_part->hdr_lines;
+  hdr = msg->this_part->hdr_lines;
   if (rprev && *rprev)
     {
       /* spool forward to the requested starting place.
